-- keys.e
--
-- dos/linux specific stuff

include misc.e          -- platform()
include graphics.e      -- cursor()
include mouse.e         -- standard mouse tools
include machine.e
include get.e
    
without warning

global integer 
    FileDelim,          -- file delimiter character
    KillBlockMode,      -- flag to turn off block mode
    BLOCK_MODE,         -- handles block code logic
    editorClockId,      -- clock routine callback
    MOUSE_COL,          -- mouse x position at time of event
    MOUSE_ROW,          -- mouse y position at time of event
    MOUSE_BUTTON_STATE  -- button held or not
    
sequence 
    cursorPosition      -- cursor position
    
object  
    MOUSE_TIME          -- time of last mouse event
    
-- initial values
    editorClockId = -1  

    mouse_events( LEFT_DOWN + LEFT_UP )     -- events to poll
    MOUSE_COL = 0
    MOUSE_ROW = 0
    MOUSE_BUTTON_STATE = LEFT_UP
    MOUSE_TIME = time()
    KillBlockMode = 0
    BLOCK_MODE = 0
    
    cursorPosition = get_position()

-- os-dependant keys
global integer
    -- editing keys
    BACKSPACE, TAB, ENTER, ESC, SHIFT_TAB,

    -- the alt keys
    ALT_A, ALT_B, ALT_C, ALT_D, ALT_E, ALT_F, ALT_G, ALT_H, ALT_I,
    ALT_J, ALT_K, ALT_L, ALT_M, ALT_N, ALT_O, ALT_P, ALT_Q, ALT_R,
    ALT_S, ALT_T, ALT_U, ALT_V, ALT_W, ALT_X, ALT_Y, ALT_Z, ALT_1,
    ALT_2, ALT_3, ALT_4, ALT_5, ALT_6, ALT_7, ALT_8, ALT_9, ALT_0,
    
    -- function keys
    F1, F2, F3, F4, F5, F6, F7, F8, F9, F10, F11, F12,

    -- navigation
    HOME, UP, PAGE_UP, LEFT, RIGHT, END, DOWN, PAGE_DOWN, INSERT, DELETE, 
    CTRL_LEFT, CTRL_RIGHT, CTRL_END, CTRL_HOME, CTRL_PAGE_UP, 
    CTRL_PAGE_DOWN, CTRL_UP, CTRL_DOWN, CTRL_INSERT,
    CTRL_TAB, CTRL_ENTER, ALT_TAB

-- os independant keys
global constant      
    CTRL_A = 01, CTRL_B = 02, CTRL_C = 03,
    CTRL_D = 04, CTRL_E = 05, CTRL_F = 06,
    CTRL_G = 07, CTRL_H = 08, CTRL_I = 09,
    CTRL_J = 10, CTRL_K = 11, CTRL_L = 12,
    CTRL_M = 13, CTRL_N = 14, CTRL_O = 15,
    CTRL_P = 16, CTRL_Q = 17, CTRL_R = 18,        
    CTRL_S = 19, CTRL_T = 20, CTRL_U = 21,
    CTRL_V = 22, CTRL_W = 23, CTRL_X = 24,
    CTRL_Y = 25, CTRL_Z = 26, 
                    
    -- emulated keys
    ALT_PRESSED = 600, ALT_RELEASED = 601,
    SHIFT_PRESSED = 603, SHIFT_RELEASED = 604,

    MOUSE_CLICK = 801, MOUSE_UP = 802, MOUSE_DRAG = 803,
    MOUSE_DOUBLE = 804, MOUSE_HELD = 805


if platform() != DOS32 then        

    -- linux returns extended keys as {27,key}
    -- i patch this by converting it to 1000+key

    -- file delimiter
    FileDelim = '/'

    -- editing keys
    BACKSPACE   = 263
    TAB         = 9 
    ENTER       = 10
    ESC         = 1027 -- was 1000
    SHIFT_TAB   = -2

    -- the alt keys
    ALT_A = 1097
    ALT_B = 1098
    ALT_C = 1099
    ALT_D = 1100
    ALT_E = 1101
    ALT_F = 1102
    ALT_G = 1103
    ALT_H = 1104
    ALT_I = 1105
    ALT_J = 1106
    ALT_K = 1107
    ALT_L = 1108
    ALT_M = 1109
    ALT_N = 1110
    ALT_O = 1111
    ALT_P = 1112
    ALT_Q = 1113    -- causes application to quit
    ALT_R = 1114
    ALT_S = 1115
    ALT_T = 1116
    ALT_U = 1117
    ALT_V = 1118
    ALT_W = 1119
    ALT_X = 1120
    ALT_Y = 1121
    ALT_Z = 1122
    ALT_0 = 1048
    ALT_1 = 1049
    ALT_2 = 1050
    ALT_3 = 1051
    ALT_4 = 1052
    ALT_5 = 1053
    ALT_6 = 1054
    ALT_7 = 1055
    ALT_8 = 1056
    ALT_9 = 1057

    -- function keys
    F1  = 265
    F2  = 266
    F3  = 267
    F4  = 268
    F5  = 269
    F6  = 270
    F7  = 271
    F8  = 272
    F9  = 273
    F10 = 274
    F11 = 275
    F12 = 276

    -- navigation keys
    UP          = 259
    DOWN        = 258
    LEFT        = 260
    RIGHT       = 261
    HOME        = 262
    END         = 360
    PAGE_UP     = 339
    PAGE_DOWN   = 338
    INSERT      = 331
    DELETE      = 330

    CTRL_LEFT       = -2
    CTRL_RIGHT      = -2
    CTRL_END        = -2
    CTRL_HOME       = -2
    CTRL_PAGE_UP    = -2
    CTRL_PAGE_DOWN  = -2
    CTRL_UP         = -2
    CTRL_DOWN       = -2
    CTRL_INSERT     = -2
    CTRL_TAB        = -2
    CTRL_ENTER      = -2
    
    ALT_TAB = 1009



else
    
    -- file delimiter
    FileDelim = '\\'

    -- editing keys
    BACKSPACE   = 8 
    TAB         = 9 
    ENTER       = 13 
    ESC         = 27
    SHIFT_TAB   = 271

    -- the alt keys
    ALT_A = 286 
    ALT_B = 304 
    ALT_C = 302
    ALT_D = 288 
    ALT_E = 274 
    ALT_F = 289
    ALT_G = 290 
    ALT_H = 291 
    ALT_I = 279
    ALT_J = 292 
    ALT_K = 293 
    ALT_L = 294
    ALT_M = 306 
    ALT_N = 305 
    ALT_O = 280
    ALT_P = 281 
    ALT_Q = 272 
    ALT_R = 275
    ALT_S = 287 
    ALT_T = 276 
    ALT_U = 278
    ALT_V = 303 
    ALT_W = 273 
    ALT_X = 301
    ALT_Y = 277 
    ALT_Z = 300 
    ALT_0 = 385
    ALT_1 = 376
    ALT_2 = 377 
    ALT_3 = 378 
    ALT_4 = 379
    ALT_5 = 380 
    ALT_6 = 381 
    ALT_7 = 382
    ALT_8 = 383 
    ALT_9 = 384 

    -- function keys
    F1  = 315 
    F2  = 316 
    F3  = 317 
    F4  = 318 
    F5  = 319    
    F6  = 320 
    F7  = 321 
    F8  = 322 
    F9  = 323 
    F10 = 324
    F11 = 389 
    F12 = 390

    -- navigation keys
    UP          = 328 
    DOWN        = 336 
    LEFT        = 331
    RIGHT       = 333 
    HOME        = 327 
    END         = 335 
    PAGE_UP     = 329 
    PAGE_DOWN   = 337 
    INSERT      = 338 
    DELETE      = 339 

    CTRL_LEFT       = 371 
    CTRL_RIGHT      = 372 
    CTRL_END        = 373
    CTRL_HOME       = 375 
    CTRL_PAGE_UP    = 388 
    CTRL_PAGE_DOWN  = 374
    CTRL_UP         = 397 
    CTRL_DOWN       = 401 
    CTRL_INSERT     = 402
    CTRL_TAB        = 404 
    CTRL_ENTER      = 10

    ALT_TAB = 421

end if
    
-- alt keys, in alpha order
global constant
    ALPHA_ALT_KEYS = { 
        ALT_A, ALT_B, ALT_C, ALT_D, ALT_E, ALT_F,
        ALT_G, ALT_H, ALT_I, ALT_J, ALT_K, ALT_L,
        ALT_M, ALT_N, ALT_O, ALT_P, ALT_Q, ALT_R,
        ALT_S, ALT_T, ALT_U, ALT_V, ALT_W, ALT_X,
        ALT_Y, ALT_Z },
    
    NUMERIC_ALT_KEYS = { 
        ALT_0, ALT_1, ALT_2, ALT_3, ALT_4,
        ALT_5, ALT_6, ALT_7, ALT_8, ALT_9 },
        
    -- block toggle key
    BLOCK_TOGGLE_KEY = ALT_B
        
    
    
-- key state stuff

-- edit mode
global integer editMode
global constant
    INSERT_MODE = 0,
    OVERWRITE_MODE = 1

global integer 
    alt_state,
    KEY_CODE,
    pendKey

-- default mode
    editMode = INSERT_MODE
    alt_state = 0
    pendKey = 0

-- clipboard
global sequence clipboard
    clipboard = ""

-- last key pressed
global integer last_key_pressed


----------------------------------------------------------------------------
global function natural_key( integer k )
        
        -- key pressed is not a command key (32-126)

        return ( k > 31 and k < 256 )

end function

----------------------------------------------------------------------------
global function shift_key_state()
    -- emulated shift state
    return BLOCK_MODE
end function


----------------------------------------------------------------------------
global function get_safe_key()
    integer key

    -- is there a pending keystroke?
    if pendKey then      
        -- return that value
        key = pendKey
        pendKey = 0

    else
    
        -- turn off block mode?
        if KillBlockMode then
            -- is it active?
            if BLOCK_MODE then
                -- send key to toggle it off
                key = BLOCK_TOGGLE_KEY
                KillBlockMode = 0
            else
                -- wait for a key
                key = wait_key()
            end if     
            
        else
            -- wait for a key
            key = wait_key()
        end if
        
        -- extended Linux key?
        if  platform() != DOS32
        and key = 27 then
            -- get the next key, add 1000 to it
            key = 1000 + wait_key()
        end if

        -- emulate a shift key
        if equal( key, BLOCK_TOGGLE_KEY ) then
            -- toggle block mode state
            BLOCK_MODE = not BLOCK_MODE

        -- is it an alt key?
        elsif find( key, ALPHA_ALT_KEYS )
        or    find( key, NUMERIC_ALT_KEYS ) then         
            -- make it look like an alt pressed
            -- on return, will feed actual key
            -- this makes the keys work with the menu,
            -- which expects an ALT_PRESSED to trigger it
            pendKey = key
            key = ALT_PRESSED
        end if
        
    end if
    
    KEY_CODE = key
    last_key_pressed = key
        
    return key
    
end function



constant Null = {}   
----------------------------------------------------------------------------
global function watch_keys()
        
    -- return an event - key press or mouse
        
    integer keyPress

    keyPress = 0
    while keyPress = 0 do

        -- run the clock, if defined
        if editorClockId > -1 then
            call_proc( editorClockId, Null )
        end if

        -- look for a key
        keyPress = get_safe_key()
                    
        -- toggle insert/delete mode on Insert
        -- if not part of block mode
        if  keyPress = INSERT
        and not BLOCK_MODE then
            if editMode = INSERT_MODE then
                editMode = OVERWRITE_MODE
            else
                editMode = INSERT_MODE
            end if
        end if

    end while

    return keyPress

end function

----------------------------------------------------------------------------
global function alt_key_state()
    return get_key() = ESC
end function


----------------------------------------------------------------------------
global function edit_mode()
        return editMode
end function

----------------------------------------------------------------------------
global procedure show_cursor()

    if editMode = INSERT_MODE then
        cursor( UNDERLINE_CURSOR )
    else
        cursor( BLOCK_CURSOR )
    end if

end procedure


----------------------------------------------------------------------------
global procedure hide_cursor()
    cursor( NO_CURSOR )
end procedure




