-- screen.e
--
-- text screen support

without warning
with trace

global integer SCREEN_WIDE      -- screen width
global integer SCREEN_DEEP      -- screen depth
global integer SCREEN_MEMORY    -- screen video memory start
-- global integer cTall, cWide

include graphics.e  -- colors
include keys.e      -- hide cursor

----------------------------------------------------------------------------

-- set resolution
--
--  text mode   = 0
--  VGA         = 18
--  SVGA        = 19 or 257

-- load support tools

----------------------------------------------------------------------------
-- Tools for deferred displaying

object result

-- attempt to set to 25 lines
    result = text_rows( 25 )
            
-- what's the configuration?
    result = video_config()

-- screen size
    SCREEN_WIDE = result[VC_COLUMNS]
    SCREEN_DEEP = result[VC_LINES] 

-- structure to hold the screen attributes
sequence
    sText, sForeColor, sBackColor

-- initialize to screen size
    sText = repeat( repeat( ' ', SCREEN_WIDE ), SCREEN_DEEP )
    sForeColor = repeat( repeat( WHITE, SCREEN_WIDE ), SCREEN_DEEP )
    sBackColor = repeat( repeat( BLACK, SCREEN_WIDE ), SCREEN_DEEP )


    
----------------------------------------------------------------------------
global procedure hide_mouse()
    -- only used in graphics mode

end procedure

----------------------------------------------------------------------------
global procedure show_mouse()
    -- only used in graphics mode

end procedure


----------------------------------------------------------------------------
global procedure writeScreen( integer row, integer col,
                              integer fColor, integer bColor, 
                              sequence text )

    -- write to the screen
    -- also updates screen data

    integer endCol

    -- misses screen?
    if col > SCREEN_WIDE then
        return
    end if

    -- calculate width
    endCol = col + length( text ) - 1
    
    -- trim to fit
    if endCol > SCREEN_WIDE then
        endCol = SCREEN_WIDE
    end if

    -- store text and attributes
    sText[row][col..endCol] = text
    sForeColor[row][col..endCol] = fColor
    sBackColor[row][col..endCol] = bColor

    -- display to screen
    text_color( fColor )
    bk_color( bColor )
    position( row, col )
    puts( 1, text )

end procedure

----------------------------------------------------------------------------
procedure restoreScreen( integer row, integer col, integer wide )

    -- redisplay line on screen data

    integer fColor, bColor, start

    -- get attributes
    fColor = sForeColor[row][col]
    bColor = sBackColor[row][col]
    start = col
    
    -- check width
    if col+wide-1 > SCREEN_WIDE then
        wide = SCREEN_WIDE-col
    end if
  
    -- read until attributes change
    for iCol = col+1 to col+wide-1 do
    
        if sForeColor[row][iCol] != fColor
        or sBackColor[row][iCol] != bColor
        then

            -- set attributes
            text_color( fColor )
            bk_color( bColor )

            -- position cursor    
            position( row, start )
    
            -- write text
            puts( 1, sText[row][start..iCol-1] )
    
            -- new start position and attributes
            start = iCol
            fColor = sForeColor[row][iCol]
            bColor = sBackColor[row][iCol]
  
        end if        
    end for

    -- set attributes
    text_color( fColor )
    bk_color( bColor )
                              
    -- position cursor    
    position( row, start )
                              
    -- write remaining text
    puts( 1, sText[row][start..col+wide-1] )

end procedure


----------------------------------------------------------------------------
global procedure put_colors( sequence colors, object text )
        
    -- given a line of text, or sequence of lines, writes them to the
    -- screen with the given attributes

    sequence cursorAt
    integer thisRow, thisCol, fColor, bColor
    integer attribute, pointer

    -- convert to a list if text is a single byte
    -- this is to save me from myself
    
    if atom( text ) then
            text = { text }
    end if

    -- skip if empty
    if length( text ) = 0 then
            return
    end if

    -- get cursor location
    cursorAt = get_position()
    thisRow = cursorAt[1]
    thisCol = cursorAt[2]

    -- get colors
    fColor = colors[1]
    bColor = colors[2]

    -- turn off screen wrapping
    wrap( 0 )

    -- if single line, convert to a list
    if atom( text[1] ) then

        -- make it an item in a list
        text = { text }

    end if

    -- display the text
    for i = 1 to length( text ) do
        
        -- out of range?
        if thisRow+i-1 > SCREEN_DEEP then
            exit
        end if          
        
        -- write to the screen
        writeScreen( thisRow+i-1, thisCol, fColor, bColor, text[i] )
    end for

    -- new row
    thisRow = thisRow + length(text) - 1
    if thisRow > SCREEN_DEEP then 
        thisRow = SCREEN_DEEP 
    end if

    -- new column
    thisCol = thisCol + length( text[length(text)] )
    if thisCol > SCREEN_WIDE then 
        thisCol = SCREEN_WIDE 
    end if

    -- reposition cursor
    position( thisRow, thisCol )


end procedure

----------------------------------------------------------------------------
global procedure put_shadow( integer wide )
    integer row, col
    sequence cursorAt
    
    -- get cursor position
    cursorAt = get_position()
    row = cursorAt[1]
    col = cursorAt[2]
    
    -- change screen colors
    sBackColor[row][col..col+wide-1] = BLACK
    sForeColor[row][col..col+wide-1] = WHITE

    -- display the screen
    restoreScreen( row, col, wide )
    
    
end procedure


----------------------------------------------------------------------------
global function save_clean_image( sequence pos1, sequence pos2 )

    integer x1, y1, x2, y2
    sequence text, fColor, bColor

    -- get position
    x1 = pos1[2]
    y1 = pos1[1]
    
    x2 = pos2[2]
    y2 = pos2[1]
    

    -- initialize
    text = {}
    fColor = {}
    bColor = {}

    -- read data
    for y = y1 to y2 do                                 
        -- accumulate
        text   = append( text,   sText[y][x1..x2] )
        fColor = append( fColor, sForeColor[y][x1..x2] )
        bColor = append( bColor, sBackColor[y][x1..x2] )
    end for
    
    return { text, fColor, bColor }

end function



----------------------------------------------------------------------------
global procedure display_clean_image( sequence pos, sequence img )

    integer x1, y1, x2, y, wide
    sequence text, fColor, bColor

    -- get position
    x1 = pos[2]
    y1 = pos[1]                  
        
    -- get values from image
    text   = img[1]
    fColor = img[2]
    bColor = img[3]

    -- width
    wide = length( text[1] )
    
    -- end column position
    x2 = x1 + wide - 1
    if x2 > SCREEN_WIDE then
        x2 = SCREEN_WIDE
    end if
    
    -- read data
    for i = 1 to length( text ) do
    
        -- calculate y position
        y = y1 + i - 1
        
        -- restore screen data
        sText[y][x1..x2]      = text[i]
        sForeColor[y][x1..x2] = fColor[i]
        sBackColor[y][x1..x2] = bColor[i]
        
        -- display screen data
        restoreScreen( y, x1, wide )
        
    end for
    
end procedure

